
from ManagerAPI.api.managers.state_manager import StateManager


def test_valid_state():
    state_manager = StateManager()
    states = [("new", True),
              ("imgresolved", True),
              ("grabbed", True),
              ("packed", True),
              ("uploaded", True),
              ("error", True),
              ("invalid", False)]

    for state in states:
        assert state_manager.valid_state(state[0]) is state[1]


def test_states():
    state_manager = StateManager()
    print(state_manager.states())
    assert state_manager.states() == {
                                      'new': 'jobs:NEW',
                                      'imgresolved': 'jobs:IMGRESOLVED',
                                      'grabbed': 'jobs:GRABBED',
                                      'packed': 'jobs:PACKED',
                                      'uploaded': 'jobs:UPLOADED',
                                      'error': 'jobs:ERROR'
                                      }



from dataclasses import dataclass
from redis import Redis
import os
import logging as log


@dataclass
class RedisConnectionConfig:
    host: str = os.getenv('REDIS_HOST') or 'localhost'
    port: int = 6379
    db: int = 1
    decode_responses: bool = True


@dataclass
class AuthManager:

    connection_config: RedisConnectionConfig = None
    refresh_token: str = ""

    def store(self):
        redis_conn = Redis(host=self.connection_config.host,
                           port=self.connection_config.port,
                           db=self.connection_config.db,
                           decode_responses=self.connection_config.decode_responses
                           )
        redis_conn.set("refresh_token", self.refresh_token)
        log.debug(f'Stored refresh token {self.refresh_token} in Redis')

        return {"refresh_token": "stored"}

    def fetch(self):
        redis_conn = Redis(host=self.connection_config.host,
                           port=self.connection_config.port,
                           db=self.connection_config.db,
                           decode_responses=self.connection_config.decode_responses
                           )
        self.refresh_token = redis_conn.get("refresh_token")
        log.debug(f'Fetched refresh token {self.refresh_token} from Redis')

        return {"refresh_token": self.refresh_token}



import uvicorn
from ManagerAPI import managerAPI
import os

HOSTNAME = os.getenv('MANAGERAPI_HOSTNAME') or 'localhost'
PORT = int(os.getenv('MANAGERAPI_PORT')) or 8000


def launch():
    config = uvicorn.Config(managerAPI, host=HOSTNAME, port=PORT, log_level="info", log_config="log.ini")
    server = uvicorn.Server(config)
    server.run()


if __name__ == '__main__':
    launch()



from LogHandler.logHandler import LogHandler
from JobHandler.jobHandler import JobHandler
from ManagerAPI.api.models.managedJob import ManagedJobModel
from VideoUploader.videoUploader.managers.upload_manager import YTAuthManager, YTVideoManager
from time import sleep, time
import logging as log


def get_video_type(job: ManagedJobModel):

    resolution = job.img_resolution.split("x")

    if resolution[0] == resolution[1]:
        return "shorts"

    return "normal"


def get_title(job: ManagedJobModel):
    title_base = f'GOES-East {job.img_date} {job.region.replace("_", " ").title()} - 24 Hour Timelapse'
    title_base_shorts = f'{job.region.replace("_", " ").title()} GOES {job.img_date} #weather'

    if get_video_type(job) == "shorts":
        return title_base_shorts

    return title_base


def get_description():
    description = """24 Hour  Timelapse

Processed and uploaded by the RootsWeather Project

https://github.com/johncanthony/RootsWeatherProject

Imagery credit to NOAA / NESDIS Center for Satellite Applications and Research.
    """

    return description


def upload_video(job: ManagedJobModel):
    log.debug(f'Uploading video for job: {job.job_id}')

    ytAuthManager = YTAuthManager().bootstrap()
    ytVideoManager = YTVideoManager(title=get_title(job=job),
                                    description=get_description(),
                                    video_type=get_video_type(job=job),
                                    privacy_status="private",
                                    video_file=job.video_urn,
                                    auth_manager=ytAuthManager)

    video_id = ytVideoManager.upload_video(ytAuthManager.fetch_credentials())
    log.info(f'Uploaded video {video_id}')

    if not video_id:
        log.error(f'Failed to upload video for job: {job.job_id}')
        job.job_status = "error"
        job.job_error += f'Failed to upload video {job.job_id}'
        return False

    job.yt_video_id = video_id

    return video_id


def run():

    jobHandler = JobHandler("packed")

    if not jobHandler.available_job():
        log.info('No jobs available')
        return

    job = jobHandler.fetch_job()

    if not job:
        log.info('No jobs available')
        return

    log.info(f'Processing job: {job.job_id}')
    upload_video(job=job)

    if job.job_status == "error":
        log.error(f'Job {job.job_id} failed')
        jobHandler.error_job(job, "Job failed")
        return

    job.job_status = "uploaded"
    job.time_end = int(time())
    jobHandler.update_job(job)


def launch():

    LogHandler(service_name="videoUploader").bootstrap()
    log.info('[Boot] Starting video uploader')

    while True:
        sleep(5)
        run()


if __name__ == "__main__":

    launch()



from JobHandler.managers.RegionURLManager import RegionURLManager


def test_getitem():
    manager = RegionURLManager()

    regionMap = [("alaska", "https://cdn.star.nesdis.noaa.gov/GOES18/ABI/SECTOR/ak/GEOCOLOR/"),
                 ("central_alaska", "https://cdn.star.nesdis.noaa.gov/GOES18/ABI/SECTOR/cak/GEOCOLOR/"),
                 ("south_mississippi_valley", "https://cdn.star.nesdis.noaa.gov/GOES16/ABI/SECTOR/smv/GEOCOLOR/"),
                 ("north_mississippi_valley", "https://cdn.star.nesdis.noaa.gov/GOES16/ABI/SECTOR/umv/GEOCOLOR/"),
                 ("pac_west", "https://cdn.star.nesdis.noaa.gov/GOES18/ABI/SECTOR/wus/GEOCOLOR/"),
                 ("mexico", "https://cdn.star.nesdis.noaa.gov/GOES16/ABI/SECTOR/mex/GEOCOLOR/"),
                 ("conus", "https://cdn.star.nesdis.noaa.gov/GOES16/ABI/CONUS/GEOCOLOR/")]

    for region in regionMap:
        assert manager[region[0]] == region[1]


def test_valid_region():
    manager = RegionURLManager()

    regions = [("alaska", True),
               ("central_alaska", True),
               ("south_mississippi_valley", True),
               ("north_mississippi_valley", True),
               ("pac_west", True), ("conus", True),
               ("invalid_region", False)]

    for region in regions:
        assert manager.valid_region(region[0]) is region[1]



from JobHandler.jobHandler import JobHandler
from JobHandler.managers.RegionURLManager import RegionURLManager
from LogHandler.logHandler import LogHandler
from ManagerAPI.api.models.managedJob import ManagedJobModel

import logging as log
from time import sleep
import requests
import os
import threading


'''
Iterate over the image links and download the images to the attached longhorn volume at /images
'''


def grab_images(job: ManagedJobModel, request_base_url: str, request_timeout: int = 10, max_threads: int = 10):

    log.info(f'Grabbing images for job: {job.job_id}')

    IMAGE_DIR = os.getenv("IMAGE_DESTINATION", "./images")

    IMAGE_DESTINATION = f'{IMAGE_DIR}/{job.job_id}/'
    if not os.path.exists(IMAGE_DESTINATION):
        os.makedirs(IMAGE_DESTINATION)

    def download_image(image_name):
        url = request_base_url + image_name
        filename = IMAGE_DESTINATION + image_name

        try:
            response = requests.get(url, timeout=request_timeout)
        except requests.exceptions.ConnectionError as err:
            log.error(f'[HTTP Connection] Unable to fetch image: [{job.job_id}] {image_name} - {err}')
            job.job_status = "error"
            job.job_error += f'Unable to fetch image: {image_name} - {err} ,'
            return
        except requests.exceptions.ReadTimeout as err:
            log.error(f'[HTTP Timeout] Unable to fetch image: [{job.job_id}] {image_name} - {err}')
            job.job_status = "error"
            job.job_error += f'Timed out fetching image: {image_name} - {err} ,'
            return

        with open(filename, 'wb') as f:
            f.write(response.content)

    threads = []

    for image_name in job.image_links.split(","):
        thread = threading.Thread(target=download_image, args=(image_name,))
        threads.append(thread)
        thread.start()

        if len(threads) >= max_threads:
            for thread in threads:
                thread.join()
            threads = []

    return


def run():

    jobHandler = JobHandler("imgresolved")
    regionalURLManager = RegionURLManager()

    log.info('Checking for new jobs')

    if not jobHandler.available_job():
        log.debug('No new jobs found')
        return

    new_job = jobHandler.fetch_job()

    if not new_job:
        log.error(f'Failed to fetch job {new_job.job_id}')
        return

    log.info(f'Fetched job {new_job.job_id}')

    if not regionalURLManager.valid_region(new_job.region):
        jobHandler.error_job(new_job, f'Invalid region: {new_job.region}')
        log.error(f'Invalid region: {new_job.region}')
        return

    grab_images(job=new_job, request_base_url=regionalURLManager[new_job.region])

    if new_job.job_status == "error":
        jobHandler.update_job(new_job)
        log.error(f'Updated job {new_job.job_id} with error')
        return

    new_job.job_status = "grabbed"
    jobHandler.update_job(new_job)
    log.debug(f'Updated job {new_job.job_id} with image links')

    return


def launch():

    LogHandler(service_name="imageGrabber").bootstrap()
    log.info('[BOOT] Starting image grabber')

    while True:
        sleep(5)
        run()


if __name__ == "__main__":

    launch()
